from ase import Atoms
from ase.io.xyz import read_xyz

from gpaw import GPAW
from gpaw.mixer import Mixer
from gpaw import ConvergenceError
from gpaw.mpi import rank

from gpaw.utilities.timing import Timer
from gpaw.eigensolvers.rmm_diis import RMM_DIIS

class MyGPAW(GPAW):
    timer_class = Timer
    
from gpaw import setup_paths

# Use setups from the $PWD and $PWD/.. first
setup_paths.insert(0, '.')
setup_paths.insert(0, '../')

atoms = read_xyz('Au102_revised.xyz')

prefix = 'Au_cluster'
L = 32.0
atoms.set_cell((L,L,L),scale_atoms=False)
atoms.center()
atoms.set_pbc(1)
r = [1, 1, 1]
atoms = atoms.repeat(r)
n = [184 * ri for ri in r]
# nbands (>=1683) is the number of bands per cluster
nbands = 3*5*16*8 # 1920
for ri in r: nbands = nbands*ri
mixer = Mixer(0.1, 5, 100.0)
# the next three lines decrease memory usage
es = RMM_DIIS(keep_htpsit=False)
from gpaw.hs_operators import Operator
Operator.nblocks = 3
calc = MyGPAW(nbands=nbands,
            # uncomment next two lines to use lcao/sz
            mode='fd',
            basis='sz',
            gpts=tuple(n),
            maxiter=20,
            width = 0.1,
            xc='LDA',
            mixer=mixer,
            eigensolver = es,
            txt=prefix + '.txt',
            )
atoms.set_calculator(calc)
try:
    pot = atoms.get_potential_energy()
except ConvergenceError:
    pass
